import sys
import os
import atexit

from PyQt5.QtWidgets import QApplication, QMainWindow, QLabel
from PyQt5.QtCore import Qt
from PyQtAds import QtAds

from perspectives import PerspectivesManager
from dockindock import DockInDockWidget
    
class MainWindow(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)
        
        self.perspectives_manager = PerspectivesManager("persist")
        self.resize(400, 400)
        self.dock_manager = DockInDockWidget(self, self.perspectives_manager, can_create_new_groups=True)
        self.setCentralWidget(self.dock_manager)
        self.dock_manager.attachViewMenu(self.menuBar().addMenu("View"))

        previous_dock_widget = None
        for i in range(3):
            l = QLabel()
            l.setWordWrap(True)
            l.setAlignment(Qt.AlignTop | Qt.AlignLeft)
            l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")
            
            previous_dock_widget = self.dock_manager.addTabWidget(l, f"Top label {i}", previous_dock_widget)
            
        last_top_level_dock = previous_dock_widget
        
        for j in range(2):
            group_manager, _ = self.dock_manager.createGroup(f"Group {j}", last_top_level_dock)
            
            previous_dock_widget = None
            
            for i in range(3):
                # Create example content label - this can be any application specific widget
                l = QLabel()
                l.setWordWrap(True)
                l.setAlignment(Qt.AlignTop | Qt.AlignLeft)
                l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")

                previous_dock_widget = group_manager.addTabWidget(l, f"ZInner {j}/{i}", previous_dock_widget)
                
            # create sub-group
            sub_group, _ = group_manager.createGroup(f"SubGroup {j}", previous_dock_widget)
            previous_dock_widget = None
            for i in range(3):
                # Create example content label - this can be any application specific widget
                l = QLabel()
                l.setWordWrap(True)
                l.setAlignment(Qt.AlignTop | Qt.AlignLeft)
                l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")

                previous_dock_widget = sub_group.addTabWidget(l, f"SubInner {j}/{i}", previous_dock_widget)
                
        self.perspectives_manager.loadPerspectives()
        
        atexit.register(self.cleanup)
        
    def cleanup(self):
        self.perspectives_manager.savePerspectives()

        
if __name__ == '__main__':
    app = QApplication(sys.argv)
    
    w = MainWindow()
    w.show()
    app.exec_()


import sys

from PyQt5.QtWidgets import (QApplication, QWidget, QVBoxLayout, QMessageBox,
                             QInputDialog, QMenu, QLineEdit)
from PyQt5.QtGui import QIcon
from PyQtAds import QtAds

from dockindockmanager import DockInDockManager
from perspectiveactions import LoadPerspectiveAction, RemovePerspectiveAction


class DockInDockWidget(QWidget):
    def __init__(self, parent, perspectives_manager: 'PerspectivesManager', can_create_new_groups: bool = False, top_level_widget = None):
        super().__init__(parent)
        
        if top_level_widget is not None:
            self.__can_create_new_groups = top_level_widget.can_create_new_groups
        else:
            self.__can_create_new_groups = can_create_new_groups
        self.__top_level_dock_widget = top_level_widget if top_level_widget else self
        self.__perspectives_manager = perspectives_manager        
        self.__new_perspective_default_name: str = ''
        
        layout = QVBoxLayout(self)
        layout.setContentsMargins(0,0,0,0)
        self.__mgr = DockInDockManager(self)
        layout.addWidget(self.__mgr)
        
    def getManager(self) -> 'DockInDockManager':
        return self.__mgr
        
    def getTopLevelDockWidget(self) -> 'DockInDockWidget':
        return self.__top_level_dock_widget
        
    def canCreateNewGroups(self) -> bool:
        return self.__can_create_new_groups
        
    def getPerspectivesManager(self) -> 'PerspectivesManager':
        return self.__perspectives_manager
        
    def addTabWidget(self, widget: QWidget, name: str, after: QtAds.CDockAreaWidget, icon = QIcon()) -> QtAds.CDockAreaWidget:
        for existing in self.getTopLevelDockWidget().getManager().allDockWidgets(True, True):
            if existing[1].objectName() == name:
                QMessageBox.critical(self, "Error", "Name '" + name + "' already in use")
                return
                
        dock_widget = QtAds.CDockWidget(name)
        dock_widget.setWidget(widget)
        dock_widget.setIcon(icon)
        
        # Add the dock widget to the top dock widget area
        return self.__mgr.addDockWidget(QtAds.CenterDockWidgetArea, dock_widget, after)
        
    def isTopLevel(self) -> bool:
        return not self.objectName()
        
    def getGroupNameError(self, group_name: str) -> str:
        if not group_name:
            return "Group must have a non-empty name"
            
        dock_managers = self.__mgr.allManagers(True, True)
        for mgr in dock_managers:
            if mgr.getGroupName() == group_name:
                return "Group name '" + group_name + "' already used"
                
        return ""
        
    def createGroup(self, group_name: str, insert_pos: QtAds.CDockAreaWidget, icon = QIcon()) -> 'DockInDockWidget':
        error = self.getGroupNameError(group_name)
        if error:
            QMessageBox.critical(None, "Error", error)
            return
            
        child = DockInDockWidget(self, self.__top_level_dock_widget, self.__perspectives_manager)
        child.setObjectName(group_name)
        
        dock_widget = QtAds.CDockWidget(group_name)
        dock_widget.setWidget(child)
        dock_widget.setIcon(icon)
        
        insert_pos = self.__mgr.addDockWidget(QtAds.CenterDockWidgetArea, dock_widget, insert_pos)

        return child, insert_pos
        
    def destroyGroup(self, widget_to_remove: 'DockInDockWidget') -> None:
        top_level_widget = widget_to_remove.getTopLevelDockWidget()
        
        if top_level_widget and top_level_widget != widget_to_remove:
            for dock_widget in widget_to_remove.getManager().getWidgetsInGUIOrder(): #don't use allDockWidgets to preserve sub-groups
                MoveDockWidgetAction.move(dock_widget, top_level_widget.getManager())
            assert not widget_to_remove.getManager().allDockWidgets(True, True)

            # find widget's parent:
            for dock_widget in top_level_widget.getManager().allDockWidgets(True, True):
                if dockwidget[1].widget() == widget_to_remove:
                    dockwidget[0].removeDockWidget(dockwidget[1])
                    del dockwidget[1]
                    # delete widgetToRemove; automatically deleted when dockWidget is deleted
                    widget_to_remove = None
                    break

            assert widget_to_remove == None
        else:
            assert False
            
    def attachViewMenu(self, menu: QMenu) -> None:
        menu.aboutToShow.connect(self.autoFillAttachedViewMenu)
        
    def autoFillAttachedViewMenu(self) -> None:
        menu = self.sender()
        
        if menu:
            menu.clear()
            self.setupViewMenu(menu)
        else:
            assert False
            
    def setupViewMenu(self, menu):
        dock_managers = self.__mgr.allManagers(True, True)
        
        has_perspectives_menu = False
        if self.getTopLevelDockWidget() == self:
            has_perspectives_menu = (self.__perspectives_manager != None)
        else:
            assert False
            
        organize = menu
        if has_perspectives_menu:
            organize = menu.addMenu("Organize")
            
        self.setupMenu(organize, dock_managers)
        
        if has_perspectives_menu:
            perspectives = menu.addMenu("Perspectives")
            self.fillPerspectivesMenu(perspectives)
            
    def setupMenu(self, menu: QMenu, move_to: 'list[DockInDockManager]') -> None:
        self.__mgr.fillViewMenu(menu, move_to)
        menu.addSeparator()
        move_menu = menu.addMenu("Move")
        self.__mgr.fillMoveMenu(move_menu, move_to)
            
    def fillPerspectivesMenu(self, menu: QMenu):
        menu.addAction("Create perspective...", self.createPerspective)
        perspectives_names = []
        if self.__perspectives_manager:
            perspectives_names = self.__perspectives_manager.perspectiveNames()
            
        if perspectives_names:
            load = menu.addMenu("Load perspective")
            for name in perspectives_names:
                load.addAction(LoadPerspectiveAction(load, name, self))
            remove = menu.addMenu("Remove perspective")
            for name in perspectives_names:
                remove.addAction(RemovePerspectiveAction(remove, name, self))
                
    def setNewPerspectiveDefaultName(default_name: str) -> None:
        self.__new_perspective_default_name = default_name
        
    def createPerspective(self) -> None:
        if not self.__perspectives_manager:
            return
            
        name = self.__new_perspective_default_name
        if self.__new_perspective_default_name:
            index = 2
            while name in self.__perspectives_manager.perspectiveNames():
                name = f"{self.__new_perspective_default_name}({index})"
                index += 1
                
        while True:
            name, ok = QInputDialog.getText(None, "Create perspective", "Enter perspective name", QLineEdit.Normal, name)
            if ok:
                if not name:
                    QMessageBox.critical(None, "Error", "Perspective name cannot be empty")
                    continue
                elif name in self.__perspectives_manager.perspectiveNames():
                    if QMessageBox.critical(None, "Error", f"Perspective '{name}' already exists, overwrite it?", QMessageBox.Yes | QMessageBox.No, QMessageBox.No) == QMessageBox.No:
                        continue
                    
                self.__perspectives_manager.addPerspective(name, self)
                break
            else:
                break
                
    def dumpStatus(self, echo: callable = print, widget: QtAds.CDockWidget = None, tab: str = '', suffix: str = '') -> str:
        if widget is not None:
            as_mgr = DockInDockManager.dockInAManager(widget)
            if as_mgr:
                as_mgr.parent().dumpStatus(tab=tab)
            else:
                echo(tab + widget.objectName() + suffix)
        else:
            echo(tab + "Group: " + self.getManager().getGroupName())
            tab += "   "
            visible_widgets = set()
            for widget in self.getManager().getWidgetsInGUIOrder():
                visible_widgets.add(widget)
                self.dumpStatus(widget=widget, tab=tab)
                
            for closed in self.getManager().dockWidgetsMap().values():
                if not closed in visible_widgets:
                    self.dumpStatus(widget=closed, tab=tab, suffix=" (closed)")


import os

from pyqtbuild import PyQtBindings, PyQtProject
from sipbuild import Option

class PyQtAds(PyQtProject):
	def __init__(self):
		""" Initialise the project. """

		super().__init__()

		self.bindings_factories = [ads]

class ads(PyQtBindings):
	def __init__(self, project):
		""" Initialise the bindings. """

		super().__init__(project, 'ads')

	def apply_user_defaults(self, tool):
		""" Set default values for user options that haven't been set yet. """

		resource_file = os.path.join(self.project.root_dir,'src','ads.qrc')
		print("Adding resource file to qmake project: ", resource_file)
		self.builder_settings.append('RESOURCES += '+resource_file)
		
		super().apply_user_defaults(tool)



import os
import sys

from PyQt5 import uic
from PyQt5.QtCore import Qt, QMargins
from PyQt5.QtWidgets import QApplication, QLabel, QVBoxLayout, QPlainTextEdit

from PyQtAds import QtAds

UI_FILE = os.path.join(os.path.dirname(__file__), 'MainWindow.ui')
MainWindowUI, MainWindowBase = uic.loadUiType(UI_FILE)


class MainWindow(MainWindowUI, MainWindowBase):

    def __init__(self, parent=None):
        super().__init__(parent)

        self.setupUi(self)
 
        # Create the dock manager. Because the parent parameter is a QMainWindow
        # the dock manager registers itself as the central widget.
        layout = QVBoxLayout(self.dockContainer);
        layout.setContentsMargins(QMargins(0, 0, 0, 0))
        self.dock_manager = QtAds.CDockManager(self.dockContainer)
        layout.addWidget(self.dock_manager)
        
        # Create example content label - this can be any application specific
        # widget
        l = QLabel()
        l.setWordWrap(True)
        l.setAlignment(Qt.AlignTop | Qt.AlignLeft);
        l.setText("Lorem ipsum dolor sit amet, consectetuer adipiscing elit. ")

        # Create a dock widget with the title Label 1 and set the created label
        # as the dock widget content
        dock_widget = QtAds.CDockWidget("Label 1")
        dock_widget.setWidget(l)

        # Add the toggleViewAction of the dock widget to the menu to give
        # the user the possibility to show the dock widget if it has been closed
        self.menuView.addAction(dock_widget.toggleViewAction())

        # Add the dock widget to the top dock widget area
        self.dock_manager.addDockWidget(QtAds.TopDockWidgetArea, dock_widget)
        
        # Create an example editor
        te = QPlainTextEdit()
        te.setPlaceholderText("Please enter your text here into this QPlainTextEdit...")
        dock_widget = QtAds.CDockWidget("Editor 1")
        self.menuView.addAction(dock_widget.toggleViewAction())
        self.dock_manager.addDockWidget(QtAds.BottomDockWidgetArea, dock_widget)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    
    w = MainWindow()
    w.show()
    app.exec_()



import sys

from PyQtAds import QtAds
from PyQt5.QtGui import QCloseEvent
from PyQt5.QtCore import (qDebug, pyqtSlot, QObject, pyqtSignal)
from PyQt5.QtWidgets import (QMainWindow, QAction, QTextEdit, QApplication,
                             QMenuBar)


class MainWindow(QMainWindow):
    dock_manager = None
    
    def closeEvent(self, event: QCloseEvent):
        super().closeEvent(event)
        if self.dock_manager is not None:
            self.dock_manager.deleteLater()
    
    def setDockManager(self, dock_manager: QtAds.CDockManager):
        self.dock_manager = dock_manager
        

if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = MainWindow()
    
    QtAds.CDockManager.setConfigFlag(QtAds.CDockManager.FocusHighlighting, True)
    QtAds.CDockManager.setConfigFlag(QtAds.CDockManager.AllTabsHaveCloseButton, True)
    dock_manager = QtAds.CDockManager(w)
    w.setDockManager(dock_manager)
    
    count = 0
    
    def on_focused_dock_widget_changed(old: QtAds.CDockWidget, now: QtAds.CDockWidget):
        global count
        qDebug( "{:d} CDockManager::focusedDockWidgetChanged old: {} now: {} visible: {}".format(
            count,
            old.objectName() if old else "-",
            now.objectName(),
            now.isVisible()))
        count += 1
        now.widget().setFocus()
        
    dock_manager.focusedDockWidgetChanged.connect(on_focused_dock_widget_changed)
    
    action = QAction("New Delete On Close", w)
    w.menuBar().addAction(action)
    
    i = 0
    def on_action_triggered():
        global i
        dw = QtAds.CDockWidget("test doc {:d}".format(i))
        i += 1
        editor = QTextEdit("lorem ipsum...", dw)
        dw.setWidget(editor)
        dw.setFeature(QtAds.CDockWidget.DockWidgetDeleteOnClose, True)
        area = dock_manager.addDockWidgetTab(QtAds.CenterDockWidgetArea, dw)
        qDebug("doc dock widget created! {} {}".format(dw, area))
    action.triggered.connect(on_action_triggered)
    
    action = QAction("New", w)
    w.menuBar().addAction(action)
    def on_action2_triggered():
        global i
        dw = QtAds.CDockWidget("test {:d}".format(i))
        i += 1
        editor = QTextEdit("lorem ipsum...", dw)
        dw.setWidget(editor)
        area = dock_manager.addDockWidgetTab(QtAds.CenterDockWidgetArea, dw)
        qDebug("dock widget created! {} {}".format(dw, area))
    action.triggered.connect(on_action2_triggered)
    
    w.show()
    app.exec_()


import os
import sys
import shlex
import shutil
import subprocess
import glob

import versioneer

from setuptools import setup, find_packages
from setuptools.command.build_py import build_py
from setuptools.extension import Extension
from distutils import sysconfig, dir_util, spawn, log, cmd
from distutils.dep_util import newer
import sipdistutils
import sipconfig
from PyQt5.QtCore import PYQT_CONFIGURATION
from PyQt5.pyrcc_main import processResourceFile

MODULE_NAME = "ads"
SRC_PATH = "PyQtAds"

REQUIRE_PYQT = True
if "--conda-recipe" in sys.argv:
    REQUIRE_PYQT = False
    sys.argv.remove("--conda-recipe")


class HostPythonConfiguration(object):
    def __init__(self):
        self.platform = sys.platform
        self.version = sys.hexversion>>8

        self.inc_dir = sysconfig.get_python_inc()
        self.venv_inc_dir = sysconfig.get_python_inc(prefix=sys.prefix)
        self.module_dir = sysconfig.get_python_lib(plat_specific=1)

        if sys.platform == 'win32':
            self.data_dir = sys.prefix
            self.lib_dir = sys.prefix +'\\libs'
        else:
            self.data_dir = sys.prefix + '/share'
            self.lib_dir = sys.prefix + '/lib'


class TargetQtConfiguration(object):
    def __init__(self, qmake):
        pipe = os.popen(' '.join([qmake, '-query']))

        for l in pipe:
            l = l.strip()

            tokens = l.split(':', 1)
            if isinstance(tokens, list):
                if len(tokens) != 2:
                    error("Unexpected output from qmake: '%s'\n" % l)

                name, value = tokens
            else:
                name = tokens
                value = None

            name = name.replace('/', '_')
            setattr(self, name, value)

        pipe.close()


class build_ext(sipdistutils.build_ext):

    description = "Builds the " + MODULE_NAME + " module."

    user_options = sipdistutils.build_ext.user_options + [
        ('qmake-bin=', None, "Path to qmake binary"),
        ('sip-bin=', None, "Path to sip binary"),
        ('qt-include-dir=', None, "Path to Qt headers"),
        ('pyqt-sip-dir=', None, "Path to PyQt's SIP files"),
        ('pyqt-sip-flags=', None, "SIP flags used to generate PyQt bindings"),
        ('sip-dir=', None, "Path to module's SIP files"),
        ('inc-dir=', None, "Path to module's include files")
    ]

    def initialize_options (self):
        super().initialize_options()
        self.qmake_bin = 'qmake'
        self.sip_bin = None
        self.qt_include_dir = None
        self.qt_libinfix = ''
        self.pyqt_sip_dir = None
        self.pyqt_sip_flags = None
        self.sip_files_dir = None
        self.sip_inc_dir = None
        self.inc_dir = None
        self.pyconfig = HostPythonConfiguration()
        self.qtconfig = TargetQtConfiguration(self.qmake_bin)
        self.config = sipconfig.Configuration()
        self.config.default_mod_dir = ("/usr/local/lib/python%i.%i/dist-packages" %
                                      (sys.version_info.major, sys.version_info.minor))

    def finalize_options (self):
        super().finalize_options()

        if not self.qt_include_dir:
            self.qt_include_dir = self.qtconfig.QT_INSTALL_HEADERS

        if not self.qt_libinfix:
            try:
                with open(os.path.join(self.qtconfig.QT_INSTALL_PREFIX, 'mkspecs', 'qconfig.pri'), 'r') as f:
                    for line in f.readlines():
                        if line.startswith('QT_LIBINFIX'):
                            self.qt_libinfix = line.split('=')[1].strip('\n').strip()
            except (FileNotFoundError, IndexError):
                pass

        if not self.pyqt_sip_dir:
            self.pyqt_sip_dir = os.path.join(self.pyconfig.data_dir, 'sip', 'PyQt5')

        if not self.pyqt_sip_flags:
            self.pyqt_sip_flags = PYQT_CONFIGURATION.get('sip_flags', '')

        if not self.sip_files_dir:
            self.sip_files_dir = os.path.abspath(os.path.join(".", "sip"))

        if not self.sip_inc_dir:
            self.sip_inc_dir = self.pyconfig.venv_inc_dir

        if not self.inc_dir:
            self.inc_dir = os.path.abspath(os.path.join(".", "src"))

        if not self.qt_include_dir:
            raise SystemExit('Could not find Qt5 headers. '
                             'Please specify via --qt-include-dir=')

        if not self.pyqt_sip_dir:
            raise SystemExit('Could not find PyQt SIP files. '
                             'Please specify containing directory via '
                             '--pyqt-sip-dir=')

        if not self.pyqt_sip_flags:
            raise SystemExit('Could not find PyQt SIP flags. '
                             'Please specify via --pyqt-sip-flags=')

    def _find_sip(self):
        """override _find_sip to allow for manually speficied sip path."""
        
        # 1. Manually specified sip_bin
        if self.sip_bin:
            return self.sip_bin

        # 2. Path determined from sipconfig.Configuration()
        #    This may not exist, depending on conda build configuration.
        sip_bin = super()._find_sip()
        if os.path.exists(sip_bin):
            return sip_bin

        # 3. Finally, fall back to sip binary found in path
        sip_bin = shutil.which('sip')
        if sip_bin:
            return sip_bin

        raise SystemExit('Could not find PyQt SIP binary.')
        
    def _sip_sipfiles_dir(self):
        sip_dir = super()._sip_sipfiles_dir()
        if os.path.exists(sip_dir):
            return sip_dir
        
        return os.path.join(sys.prefix, 'sip', 'PyQt5')

    def _sip_compile(self, sip_bin, source, sbf):
        target_dir = os.path.dirname(__file__) if self.inplace else self.build_lib
        pyi = os.path.join(target_dir, "PyQtAds", "QtAds", "ads.pyi")
        if not os.path.exists(os.path.dirname(pyi)):
            os.makedirs(os.path.dirname(pyi))
        
        cmd = [sip_bin]
        if hasattr(self, 'sip_opts'):
            cmd += self.sip_opts
        if hasattr(self, '_sip_sipfiles_dir'):
            cmd += ['-I', self._sip_sipfiles_dir()]
        cmd += [
            "-I", self.sip_files_dir,
            "-I", self.pyqt_sip_dir,
            "-I", self.sip_inc_dir,
            "-I", self.inc_dir,
            "-c", self._sip_output_dir(),
            "-b", sbf,
            "-y", pyi,
            "-w", "-o"]

        cmd += shlex.split(self.pyqt_sip_flags)  # use same SIP flags as for PyQt5
        cmd.append(source)
        self.spawn(cmd)
        
        if os.path.exists(pyi):
            with open(pyi) as f:
                content = f.readlines()
            with open(pyi, "w") as f:
                for line in content:
                    if not line.startswith("class ads"):
                        f.write(line)

    def swig_sources (self, sources, extension=None):
        if not self.extensions:
            return

        # Add the local include directory to the include path
        if extension is not None:
            extension.extra_compile_args += ['-D', 'QT_CORE_LIB',
                                             '-D', 'QT_GUI_LIB',
                                             '-D', 'QT_WIDGETS_LIB',
                                             '-D', 'ADS_SHARED_EXPORT']
            extension.include_dirs += [self.qt_include_dir, self.inc_dir,
                            os.path.join(self.qt_include_dir, 'QtCore'),
                            os.path.join(self.qt_include_dir, 'QtGui'),
                            os.path.join(self.qt_include_dir, 'QtWidgets')]
            extension.libraries += ['Qt5Core' + self.qt_libinfix,
                                    'Qt5Gui' + self.qt_libinfix,
                                    'Qt5Widgets' + self.qt_libinfix]

            if sys.platform == 'win32':
                extension.library_dirs += [self.qtconfig.QT_INSTALL_LIBS,
                                       self.inc_dir, self._sip_output_dir()]
            elif sys.platform == 'darwin':
                extension.extra_compile_args += ['-F' + self.qtconfig.QT_INSTALL_LIBS,
                    '-std=c++11', '-stdlib=libc++', '-mmacosx-version-min=10.9']
                extension.extra_link_args += ['-F' + self.qtconfig.QT_INSTALL_LIBS,
                    '-mmacosx-version-min=10.9']
            elif sys.platform == 'linux':
                extension.extra_compile_args += ['-std=c++11']

        return super().swig_sources(sources, extension)

    def build_extension(self, ext):        
        cppsources = [source for source in ext.sources if source.endswith(".cpp")]
        headersources = ['src/DockAreaTitleBar_p.h']

        dir_util.mkpath(self.build_temp, dry_run=self.dry_run)

        def get_moc_args(out_file, source):
            if sys.platform.startswith('linux'):
                return ["moc", "-D", "Q_OS_LINUX=1", "-o", out_file, source]
            if sys.platform.startswith('darwin'):
                return ["moc", "-D", "Q_OS_MACOS=1", "-o", out_file, source]
            if sys.platform.startswith('win'):
                return ["moc", "-D", "Q_OS_WIN=1", "-o", out_file, source]
            return ["moc", "-o", out_file, source]

        # Run moc on all header files.
        for source in cppsources:
            # *.cpp -> *.moc
            moc_file = os.path.basename(source).replace(".cpp", ".moc")
            out_file = os.path.join(self.build_temp, moc_file)

            if newer(source, out_file) or self.force:
                spawn.spawn(get_moc_args(out_file, source), dry_run=self.dry_run)

            header = source.replace(".cpp", ".h")
            if os.path.exists(header):
                # *.h -> moc_*.cpp
                moc_file = "moc_" + os.path.basename(header).replace(".h", ".cpp")
                out_file = os.path.join(self.build_temp, moc_file)

                if newer(header, out_file) or self.force:
                    spawn.spawn(get_moc_args(out_file, header), dry_run=self.dry_run)

                if os.path.getsize(out_file) > 0:
                    ext.sources.append(out_file)

        # Run moc on all orphan header files.
        for source in headersources:
            # *.cpp -> *.moc
            moc_file = os.path.basename(source).replace(".h", ".moc")
            out_file = os.path.join(self.build_temp, moc_file)

            if newer(source, out_file) or self.force:
                spawn.spawn(get_moc_args(out_file, source),
                            dry_run=self.dry_run)

            header = source
            if os.path.exists(header):
                # *.h -> moc_*.cpp
                moc_file = "moc_" + os.path.basename(header).replace(
                    ".h", ".cpp")
                out_file = os.path.join(self.build_temp, moc_file)

                if newer(header, out_file) or self.force:
                    spawn.spawn(get_moc_args(out_file, header),
                                dry_run=self.dry_run)

                if os.path.getsize(out_file) > 0:
                    ext.sources.append(out_file)

        # Add the temp build directory to include path, for compiler to find
        # the created .moc files
        ext.include_dirs += [self._sip_output_dir()]
        
        # Run rcc on all resource files
        resources = [source for source in ext.sources if source.endswith(".qrc")]
        for source in resources:
            ext.sources.remove(source)
            out_file = os.path.join(self.build_temp, "qrc_" + os.path.basename(source).replace(".qrc", ".cpp"))
            if newer(header, out_file) or self.force:
                spawn.spawn(["rcc", "-name", os.path.splitext(os.path.basename(source))[0], source, "-o", out_file], dry_run=self.dry_run)

            if os.path.getsize(out_file) > 0:
                ext.sources.append(out_file)

        sipdistutils.build_ext.build_extension(self, ext)
        
        import inspect
        sys.path.append(os.path.join(self.build_lib, 'PyQtAds', 'QtAds'))
        import ads

        with open(os.path.join(self.build_lib, 'PyQtAds', 'QtAds', '__init__.py'), 'w') as f:
            f.write('from .._version import *\n')
            f.write('from .ads import ads\n')
            for name, member in sorted(inspect.getmembers(ads.ads)):
                if not name.startswith('_'):
                    f.write('{0} = ads.{0}\n'.format(name))


class ProcessResourceCommand(cmd.Command):
    """A custom command to compile the resource file into a Python file"""

    description = "Compile the qrc file into a python file"

    def initialize_options(self):
        return

    def finalize_options(self):
        return

    def run(self):
        processResourceFile([os.path.join('src', 'ads.qrc')],
                            os.path.join(SRC_PATH, 'rc.py'), False)


class BuildPyCommand(build_py):
    """Custom build command to include ProcessResource command"""

    def run(self):
        self.run_command("process_resource")
        build_py.run(self)


setup_requires = ["PyQt5"] if REQUIRE_PYQT else []
cpp_sources = glob.glob(os.path.join('src', '*.cpp'))
sip_sources = [os.path.join('sip', MODULE_NAME + '.sip')]
resources = [os.path.join('src', MODULE_NAME + '.qrc')]
if sys.platform == 'linux':
    cpp_sources += glob.glob(os.path.join('src', 'linux', '*.cpp'))
ext_modules = [Extension('PyQtAds.QtAds.ads', cpp_sources + sip_sources + resources)]

install_requires = ["PyQt5"]
if sys.platform == 'win32':
    install_requires.append("pywin32")


with open('README.md', 'r') as f:
    LONG_DESCRIPTION = f.read()

setup(
    name = SRC_PATH,
    author = "Nicolas Elie",
    author_email = "nicolas.elie@cnrs.fr",
    url = "https://github.com/githubuser0xFFFF/Qt-Advanced-Docking-System",
    version = versioneer.get_version(),
    description = "Advanced Docking System for Qt",
    long_description = LONG_DESCRIPTION,
    keywords = ["qt"],
    license = "LGPLv2+",
    classifiers = ["Development Status :: 4 - Beta",
                   "Intended Audience :: Developers",
                   "License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)",
                   "Operating System :: OS Independent",
                   "Topic :: Software Development :: Libraries :: Python Modules",
                   "Environment :: Win32 (MS Windows)",
                   "Environment :: MacOS X",
                   "Environment :: X11 Applications :: Qt",
                   "Programming Language :: Python :: 3",
                   "Programming Language :: Python :: 3.2",
                   "Programming Language :: Python :: 3.3",
                   "Programming Language :: Python :: 3.4",
                   "Programming Language :: Python :: 3.5",
                   "Programming Language :: Python :: 3.6",
                   "Programming Language :: Python :: 3.7"],
    ext_modules = ext_modules,
    cmdclass = versioneer.get_cmdclass({'process_resource': ProcessResourceCommand,
                                        'build_py': BuildPyCommand,
                                        'build_ext': build_ext}),
    packages = find_packages(),
    setup_requires = setup_requires,
    install_requires = install_requires,
    zip_safe=False
)
